#===============================================================================
# LX
#

# The LX module is the home for utilities that don't particularly fit in one
# class.
module LX
	# version
	VERSION = '1.2'
	
	#---------------------------------------------------------------------------
	# verbose
	# @verbose = true
	#
	# def self.verbose
	#	return @verbose
	# end
	#
	# def self.verbose=(bool)
	#	return @verbose = bool ? true : false
	# end
	#
	# verbose
	#---------------------------------------------------------------------------


	#---------------------------------------------------------------------------
	# randstr
	#
	
	# characters for random string
	@@RAND_CHARS = %w(
		b c d f h m p q r s t v w x z
		B C D F H M P Q R S T V W X Z
		2 3 4 5 7 8 9
	)
	
	# length of random character string
	@@RND_CHARS_MAX = @@RAND_CHARS.length - 1
	
	# target length
	@@RND_PK_LENGTH = 20
	
	# Returns a random string of characters. The optional single param is the
	# length of the string to return. By default, the string is 20 characters
	# long.
	def self.randstr(len=@@RND_PK_LENGTH)
		# intialize rv
		rv = ''
		
		# build return string
		len.times do
			rv += @@RAND_CHARS[rand 0 .. @@RND_CHARS_MAX]
		end
		
		# freeze return value
		rv.freeze
		
		# return
		return rv
	end
	#
	# randstr
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# scope
	#
	
	# A little method that does nothing more than allow you to create a 'do'
	# block.
	def self.scope
		yield
	end
	
	# Same as LX::scope.
	def self.block
		yield
	end
	
	#
	# scope
	#---------------------------------------------------------------------------


	#---------------------------------------------------------------------------
	# deep_dup
	#
	
	# Deep suplicates a hash or array. Don't call this method directly. Call
	# Array#lx#deep_dup or Hash#lx#deep_dup.
	
	def self.deep_dup(obj, opts={})
		# $tm.hrm
		rv = nil
		
		# default options
		opts = {'done'=>{}}.merge(opts)
		
		# error if object has already been cloned
		if obj.is_a?(::Array) or obj.is_a?(::Hash)
			if opts['done'][obj.object_id]
				raise "object #{obj.object_id} has already been dupped"
			end
		end
		
		if obj.is_a?(::Array)
			rv = deep_dup_array(obj, opts)
		elsif obj.is_a?(::Hash)
			rv = deep_dup_hash(obj, opts)
		else
			begin
				rv = obj.dup
			rescue
				rv = obj
			end
		end
		
		# note object as done
		opts['done'][obj.object_id] = true
		
		# return rv
		return rv
	end
	#
	# deep_dup
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# deep_dup_hash
	#
	def self.deep_dup_hash(hsh, opts)
		# $tm.hrm
		rv = {}
		
		hsh.each do |key, val|
			rv[key.dup] = self.deep_dup(val, opts)
		end
		
		return rv
	end
	
	private_class_method :deep_dup_hash
	#
	# deep_dup_hash
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# deep_dup_array
	#
	def self.deep_dup_array(arr, opts)
		rv = []
		
		arr.each do |val|
			rv.push self.deep_dup(val, opts)
		end
		
		return rv
	end
	
	private_class_method :deep_dup_array
	#
	# deep_dup_array
	#---------------------------------------------------------------------------
end
#
# LX
#===============================================================================


#===============================================================================
# Array
#

# Adds a single method to the Array class. This is the only method that is
# added to the Array class itself. Access LX utilities through the array's new
# +lx+ method.
#
# For example, the use the move method, call the method like this:
#  arr = ['a', 'b', 'c']
#  arr.lx.move 1, 2
class Array
	# Creates a LX::Array object and returns it.
	def lx
		return LX::Array.new(self)
	end
end

# A helper class to add methods to the Array class. When you use the Array::lx
# you're creating an instance of LX::Array.
class LX::Array
	# initialize a LX::Array object. The only param is the array itself.
	def initialize(p_arr)
		@arr = p_arr
	end
	
	# Moves an array element from the old index to the new index.
	#  arr = ['a', 'b', 'c']
	#  arr.lx.move 1, 2 # => ["a", "c", "b"]
	def move(old_idx, new_idx)
		@arr.insert new_idx, @arr.slice!(1)
	end
	
	# Splits an array into an array of arrays. The param is the delimiter.
	#  arr = ['Fred', 'George', ':', 'Mary', 'Frank']
	#  arr.lx.split ':' # => [["Fred", "George"], ["Mary", "Frank"]]
	def split(exp)
		# $tm.hrm
		rv = []
		current = nil
		
		# loop through array looking for delimiter
		@arr.each do |el|
			if el == exp
				if current
					rv.push current
					current = nil
				end
			else
				current ||= []
				current.push el
			end
		end
		
		# add last current
		if current
			rv.push current
		end
		
		# return
		return rv
	end
	
	# draw
	def draw
		rv = @arr[0]
		@arr.rotate!
		return rv
	end
	
	# deep_dup
	def deep_dup
		return LX.deep_dup(@arr)
	end
end
#
# Array
#===============================================================================


#===============================================================================
# Hash
#

# Adds a single method to the Hash class. This is the only method that is
# added to the Hash class itself. Access LX utilities through the array's new
# +lx+ method.
#
# For example, the use the move method, call the method like this:
#  hsh = {'a'=>1, 'b'=>2, 'c'=>3}
class Hash
	# Creates a LX::Array object and returns it.
	def lx
		return LX::Hash.new(self)
	end
end

# A helper class to add methods to the Hash class. When you use the Hash::lx
# you're creating an instance of LX::Hash.
class LX::Hash
	# initialize a LX::Array object. The only param is the array itself.
	def initialize(p_hsh)
		@hsh = p_hsh
	end
	
	# deep_dup
	def deep_dup
		return LX.deep_dup(@hsh)
	end
end
#
# Hash
#===============================================================================


#===============================================================================
# String
#

# Adds the +lx+ method to the String class. To call LX utilities on the string,
# call its lx method. For example, call +collapse+ like this:
#  str = ' Whatever  Dude  '
#  str.lx.collapse # => "Whatever Dude"
class String
	# Creates and returns an LX::String object.
	def lx
		return LX::String.new(self)
	end
end

# Utilities for strings.
class LX::String
	#---------------------------------------------------------------------------
	# initialize
	#
	
	# Initializes a LX::String object, The single param is the string with which
	# the object will be associated.
	def initialize(p_str)
		@str = p_str
	end
	#
	# initialize
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# collapse
	#
	
	# Returns a new string in which leading and trailing whitespace have been
	# removed, and multiple internal whitespaces are collapsed into into single
	# spaces.
	#  str = ' Whatever  Dude  '
	#  str.lx.collapse # => "Whatever Dude"
	def collapse()
		rv = @str.dup
		rv.lx.collapse!
		return rv
	end
	
	# Performs a collapse on the string itself.
	def collapse!()
		@str.sub!(/\A[^[:graph:]]+/mu, '')
		@str.sub!(/[^[:graph:]]+\z/mu, '')
		@str.gsub!(/[^[:graph:]]+/mu, ' ')
	end
	#
	# collapse
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# inner_collapse
	#
	def inner_collapse
		rv = @str.clone
		rv.gsub!(/[\u00A0\s]+/imu, ' ')
		return rv
	end
	#
	# inner_collapse
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# content?, no_content?, nocontent?
	#
	
	# Returns true if the string contains a non-whitespace character.
	def hascontent?
		return @str.match(/\S/mu) ? true : false
	end
	
	# Same as hascontent?
	def has_content?
		return hascontent?
	end
	
	# Return true if the string does *not* have any non-whitespace characters.
	def nocontent?
		return !hascontent?
	end
	
	# Same as no_content?
	def no_content?
		return nocontent?
	end
	
	#
	# content?, no_content?, nocontent?
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# tmp_untaint
	#
	def tmp_untaint(rx)
		# dangerous
		if rx == LX::String::Dangerous
			tmp = @str.dup.untaint
		else
			if @str.match(rx)
				tmp = @str.dup.untaint
			else
				raise 'string-does-not-match-untaint-pattern: ' + rx.to_s
			end
		end
		
		# yield
		yield tmp
	end
	#
	# tmp_untaint
	#---------------------------------------------------------------------------
end

module LX::String::Dangerous
end
#
# String
#===============================================================================


#===============================================================================
# Integer
#
class Integer
	# Creates a LX::Array object and returns it.
	def lx
		return LX::Integer.new(self)
	end
end

class LX::Integer
	def initialize(int)
		@int = int
	end
	
	def multiple_of?(number)
		number != 0 ? @int % number == 0 : @int.zero?
	end
end
#
# Integer
#===============================================================================


#===============================================================================
# File
#
class File
	def self.lx
		return LX::File::Class.new(self)
	end
end

module LX::File
end

class LX::File::Class
	def initialize(clss)
	end
	
	# Atomically writes content to the given path. Does so by creating a temp
	# file, writing to it, then renaming the temp file to the final destination.
	def atomic_write(path_final, content)
		begin
			path_tmp = path_final + '.' + rand.to_s.sub(/\A.*\./mu, '')
			File.write path_tmp, content
			File.rename path_tmp, path_final
		ensure
			if File.exist?(path_tmp)
				File.delete path_tmp
			end
		end
	end
	
	# Creates a temporary path. Does *not* create a file. If, after the `do`
	# block, a file exists at that path, that file is deleted.
	def temp_path(opts = {})
		opts = {'delete'=>true}.merge(opts)
		
		# root
		if opts['root']
			root = opts['root'].sub(/\/*\z/mu, '/')
		else
			root = './'
		end
		
		# full path
		path = root + LX.randstr
		
		# add extension
		if opts['ext']
			ext = opts['ext']
			ext = ext.sub(/\A\.*/mu, '.')
			path += ext
		end
		
		if block_given?
			begin
				yield path
			ensure
				if opts['delete'] and File.exist?(path)
					File.delete path
				end
			end
		else
			return path
		end
	end
end
#
# File
#===============================================================================


#===============================================================================
# NilClass
#
class NilClass
	def lx
		return LX::NilClass.new(self)
	end
end

class LX::NilClass
	def initialize(obj)
	end
	
	# Always returns false.
	def hascontent?
		return false
	end
	
	# Always returns false.
	def has_content?
		return false
	end
	
	# Always returns true.
	def nocontent?
		return true
	end
	
	# Always returns true.
	def no_content?
		return true
	end
end
#
# NilClass
#===============================================================================